import 'package:cramr_ui_lib/routes/verb_selectionmode_selection_route.dart';
import 'package:cramr_ui_lib/routes/verb_studymode_selection_route.dart';
import 'package:cramr_ui_lib/widgets/verb_conjugation_list.dart';
import 'package:flutter/material.dart';

import '../ads/safe_banner_ad_widget.dart';
import '../settings/settings_controller.dart';
import '../settings/settings_view.dart';
import '../theme/custom_theme.dart';
import '../util/text_size_limiter.dart';

class RepeatSelectionRoute extends StatefulWidget {
  static const routeName = '/repeatSelectionRoute';

  final SettingsController settingsController;
  final SafeBannerAdWidget safeBannerAdWidget;

  const RepeatSelectionRoute(
      {Key? key,
      required this.settingsController,
      required this.safeBannerAdWidget})
      : super(key: key);

  @override
  State<StatefulWidget> createState() => RepeatSelectionRouteState();

  static void goToStudyModeSelection(BuildContext context) {
    Navigator.pushNamed(context, VerbStudymodeSelectionRoute.routeName);
  }
}

class RepeatSelectionRouteState extends State<RepeatSelectionRoute> {
  @override
  void initState() {
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return AnimatedBuilder(
        animation: widget.settingsController,
        builder: (BuildContext context, Widget? child) {
          return MaterialApp(
              debugShowCheckedModeBanner: false,
              themeMode: widget.settingsController.themeMode,
              theme: CustomTheme.lightTheme,
              darkTheme: ThemeData.dark(),
              home: SafeArea(
                  child: Scaffold(
                appBar: AppBar(
                  leading: IconButton(
                      key: const Key('BackKey'),
                      onPressed: () {
                        Navigator.pop(context);
                      },
                      icon: const Icon(Icons.arrow_back)),
                  actions: [_settingsIcon(context)],
                ),
                body: Column(
                  children: [
                    Expanded(
                        child: ListView(
                      children: _repeatables(),
                    )),
                    Container(
                      height: 100.0,
                      alignment: Alignment.center,
                      child: widget.safeBannerAdWidget,
                    )
                  ],
                ),
              )));
        });
  }

  IconButton _settingsIcon(BuildContext context) {
    return IconButton(
        key: const Key('settingsButton'),
        icon: const Icon(Icons.settings),
        onPressed: () {
          Navigator.pushNamed(context, SettingsView.routeName);
        });
  }

  List<Widget> _repeatables() {
    List<Widget> result = [];
    if (widget.settingsController.streakEngine
        .sessionConjugationsPerInfinitive()
        .isEmpty) {
      result.add(_nothingRepeatableScreen(context));
    } else {
      result.add(_repeatablesScreen(context));
    }
    return result;
  }

  Widget _repeatablesScreen(BuildContext context) {
    return VerbConjugationList(
        selection: widget.settingsController.cardinalitySelection,
        forwarder: () {
          Navigator.pushNamed(context, VerbStudymodeSelectionRoute.routeName);
        },
        sessionConjugations: widget.settingsController.streakEngine
            .sessionConjugationsPerInfinitive());
  }

  Widget _nothingRepeatableScreen(BuildContext context) {
    return Center(
        child: SizedBox(
            width: MediaQuery.of(context).size.width * .95,
            child: Column(
                crossAxisAlignment: CrossAxisAlignment.stretch,
                children: [
                  Padding(
                    padding: const EdgeInsets.all(8.0),
                    child: Card(
                      child: ListTile(
                        subtitle: Padding(
                          padding: const EdgeInsets.all(8.0),
                          child: Text(
                            'Je hebt nog geen werkwoorden om te herhalen',
                            textScaleFactor:
                                TextSizeLimiter.textScaleFactor(context),
                          ),
                        ),
                        leading: const Padding(
                          padding: EdgeInsets.all(8.0),
                          child: Icon(Icons.warning_amber),
                        ),
                      ),
                    ),
                  ),
                  const Divider(),
                  Padding(
                    padding: const EdgeInsets.all(8.0),
                    child: Card(
                      elevation: 10,
                      child: ListTile(
                        title: _learnSomethingNewWidget(context),
                      ),
                    ),
                  )
                ])));
  }

  Widget _learnSomethingNewWidget(BuildContext context) {
    return Padding(
      padding: const EdgeInsets.all(5.0),
      child: Column(
        children: [
          _learnSomethingNewText(context),
          _learnSomethingNewButton(context)
        ],
      ),
    );
  }

  Widget _learnSomethingNewText(BuildContext context) {
    return Padding(
      padding: const EdgeInsets.all(8.0),
      child: Text(
        'Leer iets nieuws',
        textAlign: TextAlign.center,
        textScaleFactor: TextSizeLimiter.textScaleFactor(context),
      ),
    );
  }

  Widget _learnSomethingNewButton(BuildContext context) {
    return Padding(
      padding: const EdgeInsets.all(8.0),
      child: FloatingActionButton(
          onPressed: () {
            Navigator.pushNamed(
                context, VerbSelectionmodeSelectionRoute.routeName);
          },
          child: const Icon(
            Icons.play_arrow_sharp,
            size: 25,
          )),
    );
  }
}
