import 'package:cramr_ui_lib/settings/settings_controller.dart';
import 'package:cramr_ui_lib/util/text_size_limiter.dart';
import 'package:flutter/material.dart';

/// Constructs a panel that allows for manipulating the cardinality in
/// cooperation with the [SettingsController]
class CardinalitySelectionPanel extends StatefulWidget {
  final SettingsController _settingsController;

  const CardinalitySelectionPanel(this._settingsController, {Key? key})
      : super(key: key);

  @override
  State<CardinalitySelectionPanel> createState() =>
      CardinalitySelectionPanelState();
}

class CardinalitySelectionPanelState extends State<CardinalitySelectionPanel> {
  late bool _singularValue;
  late bool _pluralValue;
  late bool _initialSingularValue;
  late bool _initialPluralValue;

  @override
  void initState() {
    super.initState();
    _singularValue =
        widget._settingsController.cardinalitySelection.singularEnabled;
    _pluralValue =
        widget._settingsController.cardinalitySelection.pluralEnabled;
    _initialSingularValue = _singularValue;
    _initialPluralValue = _pluralValue;
  }

  @override
  Widget build(BuildContext context) {
    return Card(
      child: Column(
        children: [
          SizedBox(
              height: 50,
              width: 200,
              child: ListTile(
                title: Text(
                  'Enkelvoud',
                  textScaleFactor: TextSizeLimiter.textScaleFactor(context),
                ),
                trailing: Switch(
                  key: const Key('singularToggle'),
                  value: _singularValue,
                  onChanged: (bool value) {
                    widget._settingsController.cardinalitySelection
                        .toggleSingularEnabled();
                    setState(() {
                      _singularValue = widget._settingsController
                          .cardinalitySelection.singularEnabled;
                      _pluralValue = widget._settingsController
                          .cardinalitySelection.pluralEnabled;
                      _evaluateDirtySwitch();
                    });
                  },
                ),
              )),
          SizedBox(
              height: 50,
              width: 200,
              child: ListTile(
                title: Text(
                  'Meervoud',
                  textScaleFactor: TextSizeLimiter.textScaleFactor(context),
                ),
                trailing: Switch(
                  key: const Key('pluralToggle'),
                  value: _pluralValue,
                  onChanged: (bool value) {
                    widget._settingsController.cardinalitySelection
                        .togglePluralEnabled();
                    setState(() {
                      _singularValue = widget._settingsController
                          .cardinalitySelection.singularEnabled;
                      _pluralValue = widget._settingsController
                          .cardinalitySelection.pluralEnabled;
                      _evaluateDirtySwitch();
                    });
                  },
                ),
              ))
        ],
      ),
    );
  }

  Future<void> _evaluateDirtySwitch() async {
    if ((_singularValue != _initialSingularValue) |
        (_pluralValue != _initialPluralValue)) {
      widget._settingsController.cardinalitySelection.markDirty();
      await widget._settingsController.updateCardinalitySelection();
    } else {
      widget._settingsController.cardinalitySelection.clearDirty();
    }
  }
}
