import 'package:flutter/material.dart';

import '../util/text_size_limiter.dart';

class ConjugationMetaDataCard extends StatelessWidget {
  final String infinitive;
  final String translation;
  final String mode;
  final String tense;

  const ConjugationMetaDataCard(
      {Key? key,
      required this.infinitive,
      required this.translation,
      required this.mode,
      required this.tense})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Card(
      margin: const EdgeInsets.fromLTRB(20, 20, 20, 20),
      elevation: 15,
      child: ListTile(
        title: _cardRow(context, 'werkwoord', infinitive),
        subtitle: Column(
          children: [
            _cardRow(context, 'vertaling', translation),
            _cardRow(context, 'modus', mode),
            _cardRow(context, 'tijd', tense),
          ],
        ),
      ),
    );
  }

  Row _cardRow(BuildContext context, String label, String value) {
    return Row(
      children: [
        SizedBox(
            width: labelWidth(context),
            child: Text(label,
                textAlign: TextAlign.left,
                textScaleFactor: TextSizeLimiter.textScaleFactor(context))),
        Text(
          ' : ',
          textAlign: TextAlign.left,
          textScaleFactor: TextSizeLimiter.textScaleFactor(context),
        ),
        Expanded(
          child: _wrappedText(context, value),
        )
      ],
    );
  }

  Widget _wrappedText(BuildContext context, String text) {
    List<Widget> widgets = [];
    text.split(' ').forEach((element) {
      widgets.add(Text(
        element.padRight(1),
        textAlign: TextAlign.left,
        overflow: TextOverflow.ellipsis,
        textScaleFactor: TextSizeLimiter.textScaleFactor(context),
      ));
    });
    Wrap result = Wrap(
      children: widgets,
    );
    return result;
  }

  double labelWidth(BuildContext context) =>
      MediaQuery.of(context).size.width * 0.38;
}
