import 'package:cramr_ui_lib/widgets/verb_conjugation_selector.dart';
import 'package:flutter/material.dart';

import '../model/cardinality_selection.dart';
import '../model/practice_session.dart';
import '../model/verb.dart';
import '../repo/verb_repository.dart';
import '../util/text_size_limiter.dart';

class VerbConjugationList extends StatelessWidget {
  final CardinalitySelection selection;
  final Function forwarder;
  final Map<String, Set<SessionConjugation>> sessionConjugations;

  const VerbConjugationList(
      {Key? key,
      required this.selection,
      required this.forwarder,
      required this.sessionConjugations})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Center(
        child: SizedBox(
            child: Padding(
      padding: const EdgeInsets.all(8.0),
      child: Column(
        children: [
          Column(
              children: _verbConjugationWidgets(context, selection, forwarder)),
        ],
      ),
    )));
  }

  List<Widget> _verbConjugationWidgets(BuildContext context,
      CardinalitySelection selection, Function forwarder) {
    List<Widget> result = [];
    for (String infinitive in sessionConjugations.keys) {
      result.add(Padding(
        padding: const EdgeInsets.all(8.0),
        child: Card(
            elevation: 10.0,
            child: _conjugationContainer(
                context, selection, forwarder, infinitive)),
      ));
    }
    return result;
  }

  Widget _conjugationContainer(BuildContext context,
      CardinalitySelection selection, Function forwarder, String infinitive) {
    return Padding(
        padding: const EdgeInsets.all(8.0),
        child: Column(
          key: Key("vervoeg $infinitive"),
          children: _verbConjugationSelectors(
              context, selection, forwarder, infinitive),
        ));
  }

  List<Widget> _verbConjugationSelectors(BuildContext context,
      CardinalitySelection selection, Function forwarder, String infinitive) {
    List<Widget> result = [];
    result.add(
      SizedBox(
        key: UniqueKey(),
        height: 40,
        child: Text(
          infinitive,
          key: UniqueKey(),
          style: Theme.of(context).textTheme.headline5,
          textScaleFactor: TextSizeLimiter.textScaleFactor(context),
        ),
      ),
    );
    String? verbTranslation = translation(infinitive);
    if (verbTranslation != null) {
      result.add(SizedBox(
        key: UniqueKey(),
        height: 40,
        child: Text(
          verbTranslation,
          key: UniqueKey(),
          style: Theme.of(context).textTheme.bodyText2,
          textScaleFactor: TextSizeLimiter.textScaleFactor(context),
        ),
      ));
    }
    Set<SessionConjugation>? conjugations = sessionConjugations[infinitive];
    for (SessionConjugation conjugation in conjugations!) {
      VerbConjugationSelector selector = VerbConjugationSelector(
          infinitive: infinitive,
          mode: conjugation.mode,
          tense: conjugation.tense,
          selection: selection,
          forwarder: forwarder);
      result.add(selector);
    }
    return result;
  }

  String? translation(String infinitive) {
    Verb? verb = VerbRepository.instance().verbs[infinitive];
    return verb?.translation;
  }
}
