import 'package:cramr_ui_lib/repo/verb_repository.dart';
import 'package:flutter/material.dart';
import 'package:provider/provider.dart';

import '../model/cardinality_selection.dart';
import '../model/verb.dart';
import '../model/verb_test_model.dart';
import '../util/practice_session_util.dart';

class VerbConjugationSelector extends StatelessWidget {
  final String infinitive;
  final String mode;
  final String tense;
  final CardinalitySelection selection;
  final Function forwarder;

  const VerbConjugationSelector(
      {Key? key,
      required this.infinitive,
      required this.mode,
      required this.tense,
      required this.selection,
      required this.forwarder})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Card(
      elevation: 5.0,
      child: ListTile(
        title: Text(
          '$mode $tense',
          style: Theme.of(context).textTheme.caption,
        ),
        trailing: InkWell(
          child: CircleAvatar(
              key: UniqueKey(),
              child:
                  Icon(Icons.play_arrow, key: Key('$infinitive $mode $tense'))),
          onTap: () {
            Verb? verb = VerbRepository.instance().verbs[infinitive];
            List<String>? indicative =
                verb?.conjugation(mode, tense)?.conjugation;
            prepareModelForSession(context, indicative!, verb!, mode, tense);
            forwarder.call();
          },
        ),
      ),
    );
  }

  void prepareModelForSession(BuildContext context, List<String> indicative,
      Verb verb, String? mode, String? tense) {
    final VerbTestModel model =
        Provider.of<VerbTestModel>(context, listen: false);
    PracticeSessionUtil.initialiseVerbTestModel(
        model, selection, indicative, verb, mode!, tense!);
  }
}
