import 'package:cramr_ui_lib/model/cardinality_selection.dart';
import 'package:flutter_test/flutter_test.dart';

void main() {
  group('Test constructor and getters', () {
    test('Test all true', () {
      CardinalitySelection allTrue = CardinalitySelection(true, true);
      expect(true, allTrue.singularEnabled);
      expect(true, allTrue.pluralEnabled);
    });
    test('Test singular true', () {
      CardinalitySelection singularTrue = CardinalitySelection(true, false);
      expect(true, singularTrue.singularEnabled);
      expect(false, singularTrue.pluralEnabled);
    });
    test('Test plural true', () {
      CardinalitySelection pluralTrue = CardinalitySelection(false, true);
      expect(false, pluralTrue.singularEnabled);
      expect(true, pluralTrue.pluralEnabled);
    });
  });

  group('Simple individual toggle tests', () {
    test('From both true to singular false', () {
      CardinalitySelection selection = CardinalitySelection(true, true);
      selection.toggleSingularEnabled();
      expect(false, selection.singularEnabled);
      expect(true, selection.pluralEnabled);
    });
    test('From both true to plural false', () {
      CardinalitySelection selection = CardinalitySelection(true, true);
      selection.togglePluralEnabled();
      expect(true, selection.singularEnabled);
      expect(false, selection.pluralEnabled);
    });
    test('From singular true to both true', () {
      CardinalitySelection selection = CardinalitySelection(true, false);
      selection.togglePluralEnabled();
      expect(true, selection.singularEnabled);
      expect(true, selection.pluralEnabled);
    });
    test('From plural true to both true', () {
      CardinalitySelection selection = CardinalitySelection(false, true);
      selection.toggleSingularEnabled();
      expect(true, selection.singularEnabled);
      expect(true, selection.pluralEnabled);
    });
  });

  group('Complex toggle tests (keep at least one enabled)', () {
    test('From only plural true to only singular true', () {
      CardinalitySelection selection = CardinalitySelection(false, true);
      selection.togglePluralEnabled();
      expect(true, selection.singularEnabled);
      expect(false, selection.pluralEnabled);
    });
    test('From only singular true to only plural true', () {
      CardinalitySelection selection = CardinalitySelection(true, false);
      selection.toggleSingularEnabled();
      expect(false, selection.singularEnabled);
      expect(true, selection.pluralEnabled);
    });
  });

  group('Independently test dirty flag', () {
    test('Test dirty flag and specific methods', () {
      CardinalitySelection selection = CardinalitySelection(true, false);
      expect(false, selection.isDirty);
      selection.markDirty();
      expect(true, selection.isDirty);
      selection.clearDirty();
      expect(false, selection.isDirty);
    });
  });
}
