import 'dart:convert';

import 'package:cramr_ui_lib/ads/ads_controller.dart';
import 'package:cramr_ui_lib/model/cardinality_selection.dart';
import 'package:cramr_ui_lib/model/practice_session.dart';
import 'package:cramr_ui_lib/model/streak_engine.dart';
import 'package:cramr_ui_lib/settings/client_context.dart';
import 'package:cramr_ui_lib/settings/notification_service.dart';
import 'package:cramr_ui_lib/settings/settings_controller.dart';
import 'package:cramr_ui_lib/settings/settings_service.dart';
import 'package:cramr_ui_lib/widgets/verb_conjugation_list.dart';
import 'package:flutter/material.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:google_mobile_ads/google_mobile_ads.dart';
import 'package:provider/provider.dart';
import 'package:shared_preferences/shared_preferences.dart';

const String _randomInfinitive = 'infinitive';
const String _mode = 'mode';
const String _tense = 'tense';
final List<String> _randomIndicative = [
  'exp1',
  'exp2',
  'exp3',
  'exp4',
  'exp5',
  'exp6',
];

void main() {
  group('VerbConjugationList generic tests', () {
    testWidgets('presence of one verb', (WidgetTester tester) async {
      SharedPreferences prefs = await initPrefs(true);
      SettingsController controller = SettingsController(
          SettingsService(prefs),
          ThemeMode.dark,
          prefs,
          NotificationService(),
          ClientContext(ClientApp.verbomatiqueSecondaire));
      MaterialApp app = await createMaterialApp(prefs, controller);
      AdsController? adsController = AdsController(MobileAds.instance);
      await tester.pumpWidget(MultiProvider(providers: [
        Provider<AdsController?>(
          create: (BuildContext context) {
            return adsController;
          },
        )
      ], child: app));
      Finder infinitiveFinder = find.text(_randomInfinitive);
      expect(infinitiveFinder, findsOneWidget);
      Finder conjugationFinder = find.text('$_mode $_tense');
      expect(conjugationFinder, findsOneWidget);
    });
  });
}

Future<MaterialApp> createMaterialApp(
    SharedPreferences prefs, SettingsController controller) async {
  await controller.loadSettings();
  return MaterialApp(
    home: VerbConjugationList(
        sessionConjugations: getSessionConjugations(),
        forwarder: () {},
        selection: CardinalitySelection(true, true)),
  );
}

Future<SharedPreferences> initPrefs(bool hasNoSessions) async {
  StreakEngine engine = StreakEngine();
  if (!hasNoSessions) {
    engine.addSession(PracticeSession(DateTime.now(), _mode, _tense,
        _randomIndicative, StudyMethod.coverMethod, _randomInfinitive));
    engine.addSession(PracticeSession(DateTime.now(), _mode, _tense,
        _randomIndicative, StudyMethod.coverMethod, _randomInfinitive));
  }
  SharedPreferences.setMockInitialValues(
      <String, Object>{'counter': 1, 'streakEngine': jsonEncode(engine)});
  SharedPreferences prefs = await SharedPreferences.getInstance();
  return prefs;
}

Map<String, Set<SessionConjugation>> getSessionConjugations() {
  Map<String, Set<SessionConjugation>> result =
      <String, Set<SessionConjugation>>{};
  Set<SessionConjugation> sessionConjugations = <SessionConjugation>{};
  sessionConjugations.add(SessionConjugation(_mode, _tense));
  result[_randomInfinitive] = sessionConjugations;
  return result;
}
